@@ -2,7 +2,7 @@
 #
 # (c) 2005, Arthur Corliss <corliss@digitalmages.com>
 #
-# $Id: Lockfile.pm,v 0.61 2009/03/05 00:06:32 acorliss Exp $
+# $Id: Lockfile.pm,v 0.63 2011/08/18 06:54:32 acorliss Exp $
 #
 #    This software is licensed under the same terms as Perl, itself.
 #    Please see http://dev.perl.org/licenses/ for more information.
@@ -29,12 +29,14 @@ use Paranoid::Debug qw(:all);
 use Paranoid::Filesystem;
 use Carp;
 
-($VERSION) = ( q$Revision: 0.61 $ =~ /(\d+(?:\.(\d+))+)/sm );
+($VERSION) = ( q$Revision: 0.63 $ =~ /(\d+(?:\.(\d+))+)/sm );
 
 @EXPORT      = qw(plock punlock pcloseLockfile);
 @EXPORT_OK   = qw(plock punlock pcloseLockfile);
 %EXPORT_TAGS = ( all => [qw(plock punlock pcloseLockfile)], );
 
+use constant PRIV_UMASK => 0600;
+
 #####################################################################
 #
 # Module code follows
@@ -48,10 +50,9 @@ use Carp;
 
     sub _clearLocks {
 
-        # Used primarily by the END block to cleanly
-        # close all lock files;
-        #
-        # Usage:  _clearLocks();
+        # Purpose:  Cleanly closes all lockfiles
+        # Returns:  True/false
+        # Usage:    $rv = _clearLocks();
 
         my ( $frv, $rv );
 
@@ -72,13 +73,19 @@ use Carp;
     }
 
     sub plock ($;$$) {
+
+        # Purpose:  Opens and locks the specified file.
+        # Returns:  True/false
+        # Usage:    $rv = plock( $filename );
+        # Usage:    $rv = plock( $filename, $lockType, $fileMode );
+
         my $filename = shift;
         my $type     = shift;
         my $mode     = shift;
         my $targ     = defined $type ? $type : 'undef';
         my $marg     = defined $mode ? $mode : 'undef';
         my $rv       = 0;
-        my $fd;
+        my ( $fd, $irv );
 
         # Validate arguments
         croak 'Mandatory first argument must be a defined filename'
@@ -90,36 +97,39 @@ use Carp;
         pdebug( "entering w/($filename)($targ)($marg)", PDLEVEL1 );
         pIn();
 
-        # Get the filehandle if it's already open
+        # Get the filehandle
         if ( exists $fd{$filename} ) {
+
+            # Retrieve a previously stored filehandle
             $fd = $fd{$filename};
 
-            # Open a new filehandle
         } else {
 
+            # Open a new filehandle
+            #
             # Set the default perms if needed
-            $mode = 0600 unless defined $mode;
+            $mode = PRIV_UMASK unless defined $mode;
 
-           # To avoid race conditions with multiple files opening (and
-           # overwriting) the same file, and hence doing flocks on descriptors
-           # with a different # (f#*&ing lock isn't working!) we attempt to do
-           # an exclusive open first.  If that fails, then we do reopen to get
-           # a filehandle to the (possibly) newly created file.
-            sysopen( $fd, $filename, O_RDWR | O_CREAT | O_EXCL, $mode )
+            # To avoid race conditions with multiple files opening (and
+            # overwriting) the same file, and hence doing flocks on
+            # descriptors with a different # (f#*&ing lock isn't working!)
+            # we attempt to do an exclusive open first.  If that fails, then
+            # we do reopen to get a filehandle to the (possibly) newly
+            # created file.
+            $irv = sysopen( $fd, $filename, O_RDWR | O_CREAT | O_EXCL, $mode )
                 || sysopen( $fd, $filename, O_RDWR );
 
             # Store the new filehandle
-            $fd{$filename} = $fd if defined $fd;
+            $fd{$filename} = $fd if $irv;
         }
 
         # Flock it
-        if ( defined $fd ) {
+        if ($irv) {
 
             # Assign the lock type according to $type
             $type = 'write' unless defined $type;
             $type = $type eq 'write' ? LOCK_EX : LOCK_SH;
-            $rv = 1;
-            flock $fd, $type;
+            $rv = flock $fd, $type;
         }
 
         pOut();
@@ -129,6 +139,11 @@ use Carp;
     }
 
     sub punlock ($) {
+
+        # Purpose:  Removes any existing locks on the file
+        # Returns:  True/false
+        # Usage:    $rv = punlock();
+
         my $filename = shift;
         my $rv       = 1;
 
@@ -148,6 +163,11 @@ use Carp;
     }
 
     sub pcloseLockfile ($) {
+
+        # Purpose:  Unlocks and closes the passed filename
+        # Returns:  True/false
+        # Usage:    $rv = pcloseLockfile( $filename );
+
         my $filename = shift;
         my $rv       = 1;
 
@@ -159,8 +179,8 @@ use Carp;
         pIn();
 
         if ( exists $fd{$filename} ) {
-            flock $fd{$filename}, LOCK_UN;
-            $rv = close $fd{$filename};
+            $rv = flock( $fd{$filename}, LOCK_UN )
+                and close $fd{$filename};
             delete $fd{$filename} if $rv;
         }
 
@@ -185,7 +205,7 @@ Paranoid::Lockfile - Paranoid Lockfile support
 
 =head1 VERSION
 
-$Id: Lockfile.pm,v 0.61 2009/03/05 00:06:32 acorliss Exp $
+$Id: Lockfile.pm,v 0.63 2011/08/18 06:54:32 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -221,6 +241,8 @@ exclusive write mode.
 You can pass an optional third argument which would be the lockfile
 filesystem permissions if the file is created.  The default is 0600.
 
+B<NOTE:> This function will block until the advisory lock is granted.
+
 =head2 punlock
 
   $rv = punlock($filename);
